@@ -17,6 +17,8 @@ import copy
 import datetime
 import hashlib
 import json
+from typing import ClassVar
+from typing import Sequence
 
 from twisted.internet import defer
 from twisted.python import log
@@ -68,7 +70,7 @@ def _canonicalize_event(event):
 class GerritChangeFilter(ChangeFilter):
     """This gerrit specific change filter helps creating pre-commit and post-commit builders"""
 
-    compare_attrs = ('eventtype_fn', 'gerrit_branch_fn')
+    compare_attrs: ClassVar[Sequence[str]] = ('eventtype_fn', 'gerrit_branch_fn')
 
     def __init__(
         self,
@@ -137,7 +139,7 @@ class GerritChangeSourceBase(base.ChangeSource, PullRequestMixin):
     """This source will maintain a connection to gerrit ssh server
     that will provide us gerrit events in json format."""
 
-    compare_attrs = ("gerritserver", "gerritport")
+    compare_attrs: ClassVar[Sequence[str]] = ("gerritserver", "gerritport")
     name = None
     # list of properties that are no of no use to be put in the event dict
     external_property_denylist = ["event.eventCreatedOn"]
@@ -273,7 +275,10 @@ class GerritSshStreamEventsConnector:
         @defer.inlineCallbacks
         def outLineReceived(self, line):
             if self.connector.debug:
-                log.msg(f"{self.connector.name} stdout: {line.decode('utf-8', errors='replace')}")
+                log.msg(
+                    f"{self.connector.change_source.name} "
+                    + f"stdout: {line.decode('utf-8', errors='replace')}"
+                )
 
             self.connector._append_line_for_debug(line)
             if self._output_enabled:
@@ -281,7 +286,10 @@ class GerritSshStreamEventsConnector:
 
         def errLineReceived(self, line):
             if self.connector.debug:
-                log.msg(f"{self.connector.name} stderr: {line.decode('utf-8', errors='replace')}")
+                log.msg(
+                    f"{self.connector.change_source.name} "
+                    + f"stderr: {line.decode('utf-8', errors='replace')}"
+                )
             if self._output_enabled:
                 self.connector._append_line_for_debug(line)
 
@@ -380,9 +388,7 @@ class GerritSshStreamEventsConnector:
         if self.ssh_server_alive_count_max is not None:
             options += ["-o", f"ServerAliveCountMax={self.ssh_server_alive_count_max}"]
 
-        cmd = (
-            ["ssh"] + options + [f"{self.username}@{self.gerritserver}", "-p", str(self.gerritport)]
-        )
+        cmd = ["ssh", *options, f"{self.username}@{self.gerritserver}", "-p", str(self.gerritport)]
 
         if self.identity_file is not None:
             cmd.extend(["-i", self.identity_file])
@@ -497,8 +503,8 @@ class GerritHttpEventLogPollerConnector:
 
     @defer.inlineCallbacks
     def setup(self):
-        self._http = yield httpclientservice.HTTPClientService.getService(
-            self._change_source.master, self._base_url, auth=self._auth
+        self._http = yield httpclientservice.HTTPSession(
+            self._change_source.master.httpservice, self._base_url, auth=self._auth
         )
 
     @defer.inlineCallbacks
@@ -515,11 +521,15 @@ class GerritHttpEventLogPollerConnector:
         last_event_formatted = last_event.strftime("%Y-%m-%d %H:%M:%S")
 
         if self.debug:
-            log.msg(f"{self.change_source.name}: Polling gerrit: {last_event_formatted}")
+            log.msg(f"{self._change_source.name}: Polling gerrit: {last_event_formatted}")
 
         res = yield self._http.get(
             "/plugins/events-log/events/", params={"t1": last_event_formatted}
         )
+        if res.code != 200:
+            log.msg(f'{self._change_source.name}: Polling gerrit: got HTTP error code {res.code}')
+            return
+
         lines = yield res.content()
         yield self._on_lines_received_cb(lines.splitlines())
 
@@ -527,9 +537,12 @@ class GerritHttpEventLogPollerConnector:
     def get_files(self, change, patchset):
         res = yield self._http.get(f"/changes/{change}/revisions/{patchset}/files/")
         res = yield res.content()
-
-        res = res.splitlines()[1].decode('utf8')  # the first line of every response is `)]}'`
-        return list(json.loads(res))
+        try:
+            res = res.splitlines()[1].decode('utf8')  # the first line of every response is `)]}'`
+            return list(json.loads(res))
+        except Exception as e:
+            log.err(e, 'while getting files from connector')
+            return []
 
     @defer.inlineCallbacks
     def do_poll(self):
@@ -564,7 +577,7 @@ class GerritChangeSource(GerritChangeSourceBase):
      - Gerrit HTTP and SSH APIs return events encoded identically
     """
 
-    compare_attrs = ("gerritserver", "gerritport")
+    compare_attrs: ClassVar[Sequence[str]] = ("gerritserver", "gerritport")
 
     name = None
 
@@ -577,6 +590,10 @@ class GerritChangeSource(GerritChangeSourceBase):
 
         # Events are received from stream event source continuously. If HTTP API is not available,
         # GerritChangeSource is always in this state.
+        #
+        # This variable is used to synchronize between concurrent data ingestion from poll or
+        # stream event sources. If _is_synchronized == True, then polling data is discarded.
+        # Otherwise, data from stream data source goes into _queued_stream_events.
         self._is_synchronized = True
 
         # True if SSH stream did not get events for a long time. It is unclear whether the
@@ -771,7 +788,7 @@ class GerritChangeSource(GerritChangeSourceBase):
         try:
             event = json.loads(bytes2unicode(line))
         except ValueError:
-            log.msg(f"bad json line: {line}")
+            log.msg(f"{self.name}: bad json line: {line}")
             return
 
         if not is_event_valid(event):
@@ -812,6 +829,10 @@ class GerritChangeSource(GerritChangeSourceBase):
             filtered_events.append((ts, event))
         return filtered_events
 
+    def _debug_log_polled_event(self, event):
+        line = json.dumps(event, sort_keys=True)
+        log.msg(f"{self.change_source.name} accepted polled event: {line}")
+
     @defer.inlineCallbacks
     def _lines_received_poll(self, lines):
         if self._is_synchronized and not self._stream_messages_timeout:
@@ -826,7 +847,7 @@ class GerritChangeSource(GerritChangeSourceBase):
             try:
                 event = json.loads(bytes2unicode(line))
             except ValueError:
-                log.msg(f"bad json line: {line}")
+                log.msg(f"{self.name}: bad json line: {line}")
                 continue
 
             if not is_event_valid(event):
@@ -859,11 +880,13 @@ class GerritChangeSource(GerritChangeSourceBase):
             needs_stream_restart = True
 
         if not self._queued_stream_events or max_event_ts <= self._queued_stream_events[0][0]:
-            # The events from stream source has not caught up - process all events and leave
-            # _is_synchronized as False.
+            # The events from poll source has not caught up to stream events - process all events
+            # and leave _is_synchronized as False.
 
             for ts, event in events:
                 self._record_last_second_event(event, ts)
+                if self.debug:
+                    self._debug_log_polled_event(event)
                 yield self.eventReceived(event)
 
             yield self._update_last_event_ts()
@@ -881,6 +904,8 @@ class GerritChangeSource(GerritChangeSourceBase):
         for ts, event in events:
             if ts <= first_queued_ts:
                 self._record_last_second_event(event, ts)
+                if self.debug:
+                    self._debug_log_polled_event(event)
                 yield self.eventReceived(event)
 
         i = 0
@@ -894,7 +919,8 @@ class GerritChangeSource(GerritChangeSourceBase):
             i += 1
 
         self._queued_stream_events.clear()
-        self._is_synchronized = True
+        if not needs_stream_restart:
+            self._is_synchronized = True
         yield self._update_last_event_ts()
         if needs_stream_restart:
             self._deferwaiter.add(self._stream_connector.restart())
@@ -982,7 +1008,7 @@ class GerritEventLogPoller(GerritChangeSourceBase):
             try:
                 event = json.loads(bytes2unicode(line))
             except ValueError:
-                log.msg(f"bad json line: {line}")
+                log.msg(f"{self.name}: bad json line: {line}")
                 continue
 
             if not is_event_valid(event):
