@@ -17,6 +17,8 @@
 Parse various kinds of 'CVS notify' email.
 """
 
+from __future__ import annotations
+
 import calendar
 import datetime
 import re
@@ -26,6 +28,8 @@ from email.iterators import body_line_iterator
 from email.utils import mktime_tz
 from email.utils import parseaddr
 from email.utils import parsedate_tz
+from typing import ClassVar
+from typing import Sequence
 
 from twisted.internet import defer
 from twisted.python import log
@@ -38,11 +42,11 @@ from buildbot.util.maildir import MaildirService
 
 @implementer(IChangeSource)
 class MaildirSource(MaildirService, util.ComparableMixin):
-
     """Generic base class for Maildir-based change sources"""
 
-    compare_attrs = ("basedir", "pollinterval", "prefix")
-    name = 'MaildirSource'
+    compare_attrs: ClassVar[Sequence[str]] = ("basedir", "pollInterval", "prefix")
+    # twisted is marked as typed, but doesn't specify this type correctly
+    name: str | None = 'MaildirSource'  # type: ignore[assignment]
 
     def __init__(self, maildir, prefix=None, category='', repository=''):
         super().__init__(maildir)
@@ -50,8 +54,9 @@ class MaildirSource(MaildirService, util.ComparableMixin):
         self.category = category
         self.repository = repository
         if prefix and not prefix.endswith("/"):
-            log.msg(f"MaildirSource: you probably want your prefix=('{prefix}') to end with "
-                    "a slash")
+            log.msg(
+                f"MaildirSource: you probably want your prefix=('{prefix}') to end with a slash"
+            )
 
     def describe(self):
         return f"{self.__class__.__name__} watching maildir '{self.basedir}'"
@@ -61,7 +66,8 @@ class MaildirSource(MaildirService, util.ComparableMixin):
         with self.moveToCurDir(filename) as f:
             chtuple = self.parse_file(f, self.prefix)
 
-        src, chdict = None, None
+        src = None
+        chdict = None
         if chtuple:
             src, chdict = chtuple
         if chdict:
@@ -77,16 +83,14 @@ class MaildirSource(MaildirService, util.ComparableMixin):
 class CVSMaildirSource(MaildirSource):
     name = "CVSMaildirSource"
 
-    def __init__(self, maildir, prefix=None, category='',
-                 repository='', properties=None):
+    def __init__(self, maildir, prefix=None, category='', repository='', properties=None):
         super().__init__(maildir, prefix, category, repository)
         if properties is None:
             properties = {}
         self.properties = properties
 
     def parse(self, m, prefix=None):
-        """Parse messages sent by the 'buildbot-cvs-mail' program.
-        """
+        """Parse messages sent by the 'buildbot-cvs-mail' program."""
         # The mail is sent from the person doing the checkin. Assume that the
         # local username is enough to identify them (this assumes a one-server
         # cvs-over-rsh environment rather than the server-dirs-shared-over-NFS
@@ -116,7 +120,7 @@ class CVSMaildirSource(MaildirSource):
         else:
             when = mktime_tz(dateTuple)
 
-        theTime = datetime.datetime.utcfromtimestamp(float(when))
+        theTime = datetime.datetime.fromtimestamp(float(when), datetime.timezone.utc)
         rev = theTime.strftime('%Y-%m-%d %H:%M:%S')
 
         catRE = re.compile(r'^Category:\s*(\S.*)')
@@ -131,7 +135,9 @@ class CVSMaildirSource(MaildirSource):
         updateRE = re.compile(r'^Update of:\s*(\S.*)')
         comments = ""
         branch = None
+        category = None
         cvsroot = None
+        cvsmode = None
         fileList = None
         files = []
         isdir = 0
@@ -205,7 +211,7 @@ class CVSMaildirSource(MaildirSource):
 
         if fileList is None:
             log.msg('CVSMaildirSource Mail with no files. Ignoring')
-            return None       # We don't have any files. Email not from CVS
+            return None  # We don't have any files. Email not from CVS
 
         if cvsmode == '1.11':
             # Please, no repo paths with spaces!
@@ -213,18 +219,18 @@ class CVSMaildirSource(MaildirSource):
             if m:
                 path = m.group(1)
             else:
-                log.msg(
-                    'CVSMaildirSource can\'t get path from file list. Ignoring mail')
+                log.msg('CVSMaildirSource can\'t get path from file list. Ignoring mail')
                 return None
-            fileList = fileList[len(path):].strip()
+            fileList = fileList[len(path) :].strip()
             singleFileRE = re.compile(
-                r'(.+?),(NONE|(?:\d+\.(?:\d+\.\d+\.)*\d+)),(NONE|(?:\d+\.(?:\d+\.\d+\.)*\d+))(?: |$)')  # noqa pylint: disable=line-too-long
+                r'(.+?),(NONE|(?:\d+\.(?:\d+\.\d+\.)*\d+)),(NONE|(?:\d+\.(?:\d+\.\d+\.)*\d+))(?: |$)'
+            )
         elif cvsmode == '1.12':
             singleFileRE = re.compile(
-                r'(.+?) (NONE|(?:\d+\.(?:\d+\.\d+\.)*\d+)) (NONE|(?:\d+\.(?:\d+\.\d+\.)*\d+))(?: |$)')  # noqa pylint: disable=line-too-long
+                r'(.+?) (NONE|(?:\d+\.(?:\d+\.\d+\.)*\d+)) (NONE|(?:\d+\.(?:\d+\.\d+\.)*\d+))(?: |$)'
+            )
             if path is None:
-                raise ValueError(
-                    'CVSMaildirSource cvs 1.12 require path. Check cvs loginfo config')
+                raise ValueError('CVSMaildirSource cvs 1.12 require path. Check cvs loginfo config')
         else:
             raise ValueError(f'Expected cvsmode 1.11 or 1.12. got: {cvsmode}')
 
@@ -234,10 +240,10 @@ class CVSMaildirSource(MaildirSource):
             if m:
                 curFile = path + '/' + m.group(1)
                 files.append(curFile)
-                fileList = fileList[m.end():]
+                fileList = fileList[m.end() :]
             else:
                 log.msg('CVSMaildirSource no files matched regex. Ignoring')
-                return None   # bail - we couldn't parse the files that changed
+                return None  # bail - we couldn't parse the files that changed
         # Now get comments
         while lines:
             line = lines.pop(0)
@@ -246,20 +252,24 @@ class CVSMaildirSource(MaildirSource):
         comments = comments.rstrip() + "\n"
         if comments == '\n':
             comments = None
-        return ('cvs', {
-            "author": author,
-            "committer": None,
-            "files": files,
-            "comments": comments,
-            "isdir": isdir,
-            "when": when,
-            "branch": branch,
-            "revision": rev,
-            "category": category,
-            "repository": cvsroot,
-            "project": project,
-            "properties": self.properties
-        })
+        return (
+            'cvs',
+            {
+                "author": author,
+                "committer": None,
+                "files": files,
+                "comments": comments,
+                "isdir": isdir,
+                "when": when,
+                "branch": branch,
+                "revision": rev,
+                "category": category,
+                "repository": cvsroot,
+                "project": project,
+                "properties": self.properties,
+            },
+        )
+
 
 # svn "commit-email.pl" handler.  The format is very similar to freshcvs mail;
 # here's a sample:
@@ -291,8 +301,7 @@ class SVNCommitEmailMaildirSource(MaildirSource):
     name = "SVN commit-email.pl"
 
     def parse(self, m, prefix=None):
-        """Parse messages sent by the svn 'commit-email.pl' trigger.
-        """
+        """Parse messages sent by the svn 'commit-email.pl' trigger."""
 
         # The mail is sent from the person doing the checkin. Assume that the
         # local username is enough to identify them (this assumes a one-server
@@ -357,11 +366,11 @@ class SVNCommitEmailMaildirSource(MaildirSource):
             if line == "\n":
                 break
             if line.find("Modified:\n") == 0:
-                continue            # ignore this line
+                continue  # ignore this line
             if line.find("Added:\n") == 0:
-                continue            # ignore this line
+                continue  # ignore this line
             if line.find("Removed:\n") == 0:
-                continue            # ignore this line
+                continue  # ignore this line
             line = line.strip()
 
             thesefiles = line.split(" ")
@@ -370,10 +379,12 @@ class SVNCommitEmailMaildirSource(MaildirSource):
                     # insist that the file start with the prefix: we may get
                     # changes we don't care about too
                     if f.startswith(prefix):
-                        f = f[len(prefix):]
+                        f = f[len(prefix) :]
                     else:
-                        log.msg(f"ignored file from svn commit: prefix '{prefix}' "
-                                f"does not match filename '{f}'")
+                        log.msg(
+                            f"ignored file from svn commit: prefix '{prefix}' "
+                            f"does not match filename '{f}'"
+                        )
                         continue
 
                 # TODO: figure out how new directories are described, set
@@ -384,14 +395,18 @@ class SVNCommitEmailMaildirSource(MaildirSource):
             log.msg("no matching files found, ignoring commit")
             return None
 
-        return ('svn', {
-            "author": author,
-            "committer": None,
-            "files": files,
-            "comments": comments,
-            "when": when,
-            "revision": rev
-        })
+        return (
+            'svn',
+            {
+                "author": author,
+                "committer": None,
+                "files": files,
+                "comments": comments,
+                "when": when,
+                "revision": rev,
+            },
+        )
+
 
 # bzr Launchpad branch subscription mails. Sample mail:
 #
@@ -425,7 +440,7 @@ class SVNCommitEmailMaildirSource(MaildirSource):
 class BzrLaunchpadEmailMaildirSource(MaildirSource):
     name = "Launchpad"
 
-    compare_attrs = ("branchMap", "defaultBranch")
+    compare_attrs: ClassVar[Sequence[str]] = ("branchMap", "defaultBranch")
 
     def __init__(self, maildir, prefix=None, branchMap=None, defaultBranch=None, **kwargs):
         self.branchMap = branchMap
@@ -433,8 +448,7 @@ class BzrLaunchpadEmailMaildirSource(MaildirSource):
         super().__init__(maildir, prefix, **kwargs)
 
     def parse(self, m, prefix=None):
-        """Parse branch notification messages sent by Launchpad.
-        """
+        """Parse branch notification messages sent by Launchpad."""
 
         subject = m["subject"]
         match = re.search(r"^\s*\[Branch\s+([^]]+)\]", subject)
@@ -489,14 +503,15 @@ class BzrLaunchpadEmailMaildirSource(MaildirSource):
             # datetime.strptime() is supposed to support %z for time zone, but
             # it does not seem to work. So handle the time zone manually.
             match = re.search(
-                r"^timestamp: [a-zA-Z]{3} (\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}) ([-+])(\d{2})(\d{2})$", line)  # noqa pylint: disable=line-too-long
+                r"^timestamp: [a-zA-Z]{3} (\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}) ([-+])(\d{2})(\d{2})$",
+                line,
+            )
             if match:
                 datestr = match.group(1)
                 tz_sign = match.group(2)
                 tz_hours = match.group(3)
                 tz_minutes = match.group(4)
-                when = parseLaunchpadDate(
-                    datestr, tz_sign, tz_hours, tz_minutes)
+                when = parseLaunchpadDate(datestr, tz_sign, tz_hours, tz_minutes)
 
             if re.search(r"^message:\s*$", line):
                 gobbler = gobble_comment
@@ -528,16 +543,19 @@ class BzrLaunchpadEmailMaildirSource(MaildirSource):
                     branch = None
 
         if rev and author:
-            return ('bzr', {
-                "author": author,
-                "committer": None,
-                "files": d['files'],
-                "comments": d['comments'],
-                "when": when,
-                "revision": rev,
-                "branch": branch,
-                "repository": repository or ''
-            })
+            return (
+                'bzr',
+                {
+                    "author": author,
+                    "committer": None,
+                    "files": d['files'],
+                    "comments": d['comments'],
+                    "when": when,
+                    "revision": rev,
+                    "branch": branch,
+                    "repository": repository or '',
+                },
+            )
         return None
 
 
