@@ -18,6 +18,8 @@
 
 import os
 import xml.dom.minidom
+from typing import ClassVar
+from typing import Sequence
 from urllib.parse import quote_plus as urlquote_plus
 
 from twisted.internet import defer
@@ -72,7 +74,7 @@ class SVNPoller(base.ReconfigurablePollingChangeSource, util.ComparableMixin):
     master.
     """
 
-    compare_attrs = (
+    compare_attrs: ClassVar[Sequence[str]] = (
         "repourl",
         "split_file",
         "svnuser",
@@ -173,7 +175,7 @@ class SVNPoller(base.ReconfigurablePollingChangeSource, util.ComparableMixin):
         self.cachepath = cachepath
         if self.cachepath and os.path.exists(self.cachepath):
             try:
-                with open(self.cachepath, "r", encoding='utf-8') as f:
+                with open(self.cachepath, encoding='utf-8') as f:
                     self.last_change = int(f.read().strip())
                     log.msg(
                         f"SVNPoller: SVNPoller({self.repourl}) setting last_change "
@@ -276,7 +278,7 @@ class SVNPoller(base.ReconfigurablePollingChangeSource, util.ComparableMixin):
         )
 
         if rc != 0:
-            raise EnvironmentError(f'{self}: Got error when retrieving svn prefix')
+            raise OSError(f'{self}: Got error when retrieving svn prefix')
 
         try:
             doc = xml.dom.minidom.parseString(output)
@@ -326,7 +328,7 @@ class SVNPoller(base.ReconfigurablePollingChangeSource, util.ComparableMixin):
         )
 
         if rc != 0:
-            raise EnvironmentError(f'{self}: Got error when retrieving svn logs')
+            raise OSError(f'{self}: Got error when retrieving svn logs')
 
         return output
 
