@@ -11,8 +11,11 @@
 # this program; if not, write to the Free Software Foundation, Inc., 51
 # Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
-# Copyright Buildbot Team Members
+# Copyright Buildbot Team Member
 
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
 
 from twisted.internet import defer
 
@@ -21,17 +24,20 @@ from buildbot.data import types
 from buildbot.db.logs import LogSlugExistsError
 from buildbot.util import identifiers
 
+if TYPE_CHECKING:
+    from buildbot.db.logs import LogModel
+
 
 class EndpointMixin:
-    def db2data(self, dbdict):
+    def db2data(self, model: LogModel):
         data = {
-            'logid': dbdict['id'],
-            'name': dbdict['name'],
-            'slug': dbdict['slug'],
-            'stepid': dbdict['stepid'],
-            'complete': dbdict['complete'],
-            'num_lines': dbdict['num_lines'],
-            'type': dbdict['type'],
+            'logid': model.id,
+            'name': model.name,
+            'slug': model.slug,
+            'stepid': model.stepid,
+            'complete': model.complete,
+            'num_lines': model.num_lines,
+            'type': model.type,
         }
         return defer.succeed(data)
 
@@ -45,8 +51,8 @@ class LogEndpoint(EndpointMixin, base.BuildNestingMixin, base.Endpoint):
         /builds/n:buildid/steps/n:step_number/logs/i:log_slug
         /builders/n:builderid/builds/n:build_number/steps/i:step_name/logs/i:log_slug
         /builders/n:builderid/builds/n:build_number/steps/n:step_number/logs/i:log_slug
-        /builders/i:buildername/builds/n:build_number/steps/i:step_name/logs/i:log_slug
-        /builders/i:buildername/builds/n:build_number/steps/n:step_number/logs/i:log_slug
+        /builders/s:buildername/builds/n:build_number/steps/i:step_name/logs/i:log_slug
+        /builders/s:buildername/builds/n:build_number/steps/n:step_number/logs/i:log_slug
     """
 
     @defer.inlineCallbacks
@@ -60,7 +66,7 @@ class LogEndpoint(EndpointMixin, base.BuildNestingMixin, base.Endpoint):
         if step_dict is None:
             return None
 
-        dbdict = yield self.master.db.logs.getLogBySlug(step_dict['id'], kwargs.get('log_slug'))
+        dbdict = yield self.master.db.logs.getLogBySlug(step_dict.id, kwargs.get('log_slug'))
         return (yield self.db2data(dbdict)) if dbdict else None
 
 
@@ -72,8 +78,8 @@ class LogsEndpoint(EndpointMixin, base.BuildNestingMixin, base.Endpoint):
         /builds/n:buildid/steps/n:step_number/logs
         /builders/n:builderid/builds/n:build_number/steps/i:step_name/logs
         /builders/n:builderid/builds/n:build_number/steps/n:step_number/logs
-        /builders/i:buildername/builds/n:build_number/steps/i:step_name/logs
-        /builders/i:buildername/builds/n:build_number/steps/n:step_number/logs
+        /builders/s:buildername/builds/n:build_number/steps/i:step_name/logs
+        /builders/s:buildername/builds/n:build_number/steps/n:step_number/logs
     """
 
     @defer.inlineCallbacks
@@ -82,7 +88,7 @@ class LogsEndpoint(EndpointMixin, base.BuildNestingMixin, base.Endpoint):
         step_dict = yield retriever.get_step_dict()
         if step_dict is None:
             return []
-        logs = yield self.master.db.logs.getLogs(stepid=step_dict['id'])
+        logs = yield self.master.db.logs.getLogs(stepid=step_dict.id)
         results = []
         for dbdict in logs:
             results.append((yield self.db2data(dbdict)))
