@@ -13,22 +13,52 @@
 #
 # Copyright Buildbot Team Members
 
+from __future__ import annotations
+
+from dataclasses import dataclass
 
 import sqlalchemy as sa
 from twisted.internet import defer
 
 from buildbot.db import NULL
 from buildbot.db import base
+from buildbot.util.sautils import hash_columns
+from buildbot.warnings import warn_deprecated
 
 
 class ChangeSourceAlreadyClaimedError(Exception):
     pass
 
 
+@dataclass
+class ChangeSourceModel:
+    id: int
+    name: str
+
+    masterid: int | None = None
+
+    # For backward compatibility
+    def __getitem__(self, key: str):
+        warn_deprecated(
+            '4.1.0',
+            (
+                'ChangeSourcesConnectorComponent '
+                'getChangeSource, and getChangeSources '
+                'no longer return ChangeSource as dictionnaries. '
+                'Usage of [] accessor is deprecated: please access the member directly'
+            ),
+        )
+
+        if hasattr(self, key):
+            return getattr(self, key)
+
+        raise KeyError(key)
+
+
 class ChangeSourcesConnectorComponent(base.DBConnectorComponent):
     def findChangeSourceId(self, name):
         tbl = self.db.model.changesources
-        name_hash = self.hashColumns(name)
+        name_hash = hash_columns(name)
         return self.findSomethingId(
             tbl=tbl,
             whereclause=(tbl.c.name_hash == name_hash),
@@ -44,18 +74,35 @@ class ChangeSourcesConnectorComponent(base.DBConnectorComponent):
             if masterid is None:
                 q = cs_mst_tbl.delete().where(cs_mst_tbl.c.changesourceid == changesourceid)
                 conn.execute(q)
+                conn.commit()
                 return
 
             # try a blind insert..
             try:
                 q = cs_mst_tbl.insert()
                 conn.execute(q, {"changesourceid": changesourceid, "masterid": masterid})
+                conn.commit()
             except (sa.exc.IntegrityError, sa.exc.ProgrammingError) as e:
+                conn.rollback()
                 # someone already owns this changesource.
                 raise ChangeSourceAlreadyClaimedError from e
 
         return self.db.pool.do(thd)
 
+    def get_change_source_master(self, changesourceid):
+        def thd(conn):
+            q = sa.select(self.db.model.changesource_masters.c.masterid).where(
+                self.db.model.changesource_masters.c.changesourceid == changesourceid
+            )
+            r = conn.execute(q)
+            row = r.fetchone()
+            conn.close()
+            if row:
+                return row.masterid
+            return None
+
+        return self.db.pool.do(thd)
+
     @defer.inlineCallbacks
     def getChangeSource(self, changesourceid):
         cs = yield self.getChangeSources(_changesourceid=changesourceid)
@@ -96,9 +143,9 @@ class ChangeSourcesConnectorComponent(base.DBConnectorComponent):
             if wc is not None:
                 q = q.where(wc)
 
-            return [
-                {"id": row.id, "name": row.name, "masterid": row.masterid}
-                for row in conn.execute(q).fetchall()
-            ]
+            return [self._model_from_row(row) for row in conn.execute(q).fetchall()]
 
         return self.db.pool.do(thd)
+
+    def _model_from_row(self, row):
+        return ChangeSourceModel(id=row.id, name=row.name, masterid=row.masterid)
