@@ -25,12 +25,13 @@ from twisted.python import log
 from twisted.python import util
 
 from buildbot.db import base
+from buildbot.db import model_config
 from buildbot.db.migrate_utils import test_unicode
 from buildbot.db.types.json import JsonObject
 from buildbot.util import sautils
 
 if TYPE_CHECKING:
-    from sqlalchemy.engine.base import Connectable as SQLAConnection
+    from sqlalchemy.engine import Connection as SQLAConnection
     from sqlalchemy.engine.reflection import Inspector
 
 
@@ -53,14 +54,6 @@ class UpgradeFromBefore3p0Error(Exception):
 
 
 class Model(base.DBConnectorComponent):
-    property_name_length = 256
-    property_source_length = 256
-    hash_length = 40
-
-    #
-    # schema
-    #
-
     metadata = sa.MetaData()
 
     # NOTES
@@ -147,10 +140,10 @@ class Model(base.DBConnectorComponent):
         sa.Column(
             'buildid', sa.Integer, sa.ForeignKey('builds.id', ondelete='CASCADE'), nullable=False
         ),
-        sa.Column('name', sa.String(property_name_length), nullable=False),
+        sa.Column('name', sa.String(model_config.property_name_length), nullable=False),
         # JSON encoded value
         sa.Column('value', sa.Text, nullable=False),
-        sa.Column('source', sa.String(property_source_length), nullable=False),
+        sa.Column('source', sa.String(model_config.property_source_length), nullable=False),
     )
 
     # This table contains transient build state.
@@ -265,7 +258,7 @@ class Model(base.DBConnectorComponent):
         sa.Column('first_line', sa.Integer, nullable=False),
         sa.Column('last_line', sa.Integer, nullable=False),
         # log contents, including a terminating newline, encoded in utf-8 or,
-        # if 'compressed' is not 0, compressed with gzip, bzip2 or lz4
+        # if 'compressed' is not 0, compressed with gzip, bzip2, lz4, br or zstd
         sa.Column('content', sa.LargeBinary(65536)),
         sa.Column('compressed', sa.SmallInteger, nullable=False),
     )
@@ -283,7 +276,7 @@ class Model(base.DBConnectorComponent):
             sa.ForeignKey('buildsets.id', ondelete='CASCADE'),
             nullable=False,
         ),
-        sa.Column('property_name', sa.String(property_name_length), nullable=False),
+        sa.Column('property_name', sa.String(model_config.property_name_length), nullable=False),
         # JSON-encoded tuple of (value, source)
         sa.Column('property_value', sa.Text, nullable=False),
     )
@@ -341,7 +334,7 @@ class Model(base.DBConnectorComponent):
         # name for this changesource, as given in the configuration, plus a hash
         # of that name used for a unique index
         sa.Column('name', sa.Text, nullable=False),
-        sa.Column('name_hash', sa.String(hash_length), nullable=False),
+        sa.Column('name_hash', sa.String(model_config.hash_length), nullable=False),
     )
 
     # This links changesources to the master where they are running.  A changesource
@@ -434,7 +427,7 @@ class Model(base.DBConnectorComponent):
             sa.ForeignKey('changes.changeid', ondelete='CASCADE'),
             nullable=False,
         ),
-        sa.Column('property_name', sa.String(property_name_length), nullable=False),
+        sa.Column('property_name', sa.String(model_config.property_name_length), nullable=False),
         # JSON-encoded tuple of (value, source)
         sa.Column('property_value', sa.Text, nullable=False),
     )
@@ -537,8 +530,8 @@ class Model(base.DBConnectorComponent):
         metadata,
         sa.Column('id', sa.Integer, primary_key=True),
         # hash of the branch, revision, patchid, repository, codebase, and
-        # project, using hashColumns.
-        sa.Column('ss_hash', sa.String(hash_length), nullable=False),
+        # project, using hash_columns.
+        sa.Column('ss_hash', sa.String(model_config.hash_length), nullable=False),
         # the branch to check out.  When branch is NULL, that means
         # the main branch (trunk, master, etc.)
         sa.Column('branch', sa.String(256)),
@@ -591,7 +584,7 @@ class Model(base.DBConnectorComponent):
         # name for this scheduler, as given in the configuration, plus a hash
         # of that name used for a unique index
         sa.Column('name', sa.Text, nullable=False),
-        sa.Column('name_hash', sa.String(hash_length), nullable=False),
+        sa.Column('name_hash', sa.String(model_config.hash_length), nullable=False),
         sa.Column('enabled', sa.SmallInteger, server_default=sa.DefaultClause("1")),
     )
 
@@ -650,7 +643,7 @@ class Model(base.DBConnectorComponent):
         # project name
         sa.Column('name', sa.Text, nullable=False),
         # sha1 of name; used for a unique index
-        sa.Column('name_hash', sa.String(hash_length), nullable=False),
+        sa.Column('name_hash', sa.String(model_config.hash_length), nullable=False),
         # project slug, potentially shown in the URLs
         sa.Column('slug', sa.String(50), nullable=False),
         # project description
@@ -684,7 +677,7 @@ class Model(base.DBConnectorComponent):
             nullable=True,
         ),
         # sha1 of name; used for a unique index
-        sa.Column('name_hash', sa.String(hash_length), nullable=False),
+        sa.Column('name_hash', sa.String(model_config.hash_length), nullable=False),
     )
 
     # This links builders to the master where they are running.  A builder
@@ -715,7 +708,7 @@ class Model(base.DBConnectorComponent):
         # tag's name
         sa.Column('name', sa.Text, nullable=False),
         # sha1 of name; used for a unique index
-        sa.Column('name_hash', sa.String(hash_length), nullable=False),
+        sa.Column('name_hash', sa.String(model_config.hash_length), nullable=False),
     )
 
     # a many-to-may relationship between builders and tags
@@ -925,7 +918,7 @@ class Model(base.DBConnectorComponent):
         # master's name (generally in the form hostname:basedir)
         sa.Column('name', sa.Text, nullable=False),
         # sha1 of name; used for a unique index
-        sa.Column('name_hash', sa.String(hash_length), nullable=False),
+        sa.Column('name_hash', sa.String(model_config.hash_length), nullable=False),
         # true if this master is running
         sa.Column('active', sa.Integer, nullable=False),
         # updated periodically by a running master, so silently failed masters
@@ -1146,6 +1139,7 @@ class Model(base.DBConnectorComponent):
     def alembic_stamp(self, conn, alembic_scripts, revision):
         context = alembic.runtime.migration.MigrationContext.configure(conn)
         context.stamp(alembic_scripts, revision)
+        conn.commit()
 
     @defer.inlineCallbacks
     def is_current(self):
@@ -1194,6 +1188,7 @@ class Model(base.DBConnectorComponent):
 
                 self.alembic_stamp(conn, alembic_scripts, alembic_scripts.get_base())
                 conn.execute(sa.text('drop table migrate_version'))
+                conn.commit()
 
             if not self.table_exists(conn, 'alembic_version'):
                 log.msg("Initializing empty database")
@@ -1202,6 +1197,7 @@ class Model(base.DBConnectorComponent):
                 test_unicode(conn)
 
                 Model.metadata.create_all(conn)
+                conn.commit()
                 self.alembic_stamp(conn, alembic_scripts, current_script_rev_head)
                 return
 
