@@ -22,7 +22,6 @@ from twisted.application import internet
 from twisted.internet import defer
 from twisted.internet import task
 from twisted.internet import threads
-from twisted.python import failure
 from twisted.python import log
 
 import buildbot
@@ -46,6 +45,7 @@ from buildbot.process.users.manager import UserManagerManager
 from buildbot.schedulers.manager import SchedulerManager
 from buildbot.secrets.manager import SecretManager
 from buildbot.util import check_functional_environment
+from buildbot.util import httpclientservice
 from buildbot.util import service
 from buildbot.util.eventual import eventually
 from buildbot.wamp import connector as wampconnector
@@ -137,6 +137,7 @@ class BuildMaster(service.ReconfigurableServiceMixin, service.MasterService):
     def create_child_services(self):
         # note that these are order-dependent.  If you get the order wrong,
         # you'll know it, as the master will fail to start.
+        self.httpservice = yield httpclientservice.HTTPClientService.getService(self, '')
 
         self.metrics = metrics.MetricLogObserver()
         yield self.metrics.setServiceParent(self)
@@ -169,7 +170,7 @@ class BuildMaster(service.ReconfigurableServiceMixin, service.MasterService):
         yield self.user_manager.setServiceParent(self)
 
         self.db = dbconnector.DBConnector(self.basedir)
-        yield self.db.setServiceParent(self)
+        yield self.db.set_master(self)
 
         self.wamp = wampconnector.WampConnector()
         yield self.wamp.setServiceParent(self)
@@ -191,7 +192,7 @@ class BuildMaster(service.ReconfigurableServiceMixin, service.MasterService):
 
         self.secrets_manager = SecretManager()
         yield self.secrets_manager.setServiceParent(self)
-        self.secrets_manager.reconfig_priority = 2000
+        self.secrets_manager.reconfig_priority = self.db.reconfig_priority - 1
 
         self.service_manager = service.BuildbotServiceManager()
         yield self.service_manager.setServiceParent(self)
@@ -258,13 +259,14 @@ class BuildMaster(service.ReconfigurableServiceMixin, service.MasterService):
                 log.msg("Halting master.")
                 self.reactor.stop()
                 return
-            except Exception:
-                log.err(failure.Failure(), 'while starting BuildMaster')
+            except Exception as e:
+                log.err(e, 'while starting BuildMaster')
                 self.reactor.stop()
                 return
 
             # set up services that need access to the config before everything
             # else gets told to reconfig
+            yield self.secrets_manager.setup()
             try:
                 yield self.db.setup()
             except exceptions.DatabaseNotReadyError:
@@ -272,6 +274,8 @@ class BuildMaster(service.ReconfigurableServiceMixin, service.MasterService):
                 self.reactor.stop()
                 return
 
+            yield self.db.startService()
+
             yield self.mq.setup()
 
             # the buildbot scripts send the SIGHUP signal to reconfig master
@@ -320,9 +324,8 @@ class BuildMaster(service.ReconfigurableServiceMixin, service.MasterService):
             # send the statistics to buildbot.net, without waiting
             self.sendBuildbotNetUsageData()
             startup_succeed = True
-        except Exception:
-            f = failure.Failure()
-            log.err(f, 'while starting BuildMaster')
+        except Exception as e:
+            log.err(e, 'while starting BuildMaster')
             self.reactor.stop()
 
         finally:
@@ -372,6 +375,8 @@ class BuildMaster(service.ReconfigurableServiceMixin, service.MasterService):
             self._master_initialized = False
         finally:
             yield self.initLock.release()
+            if self.db.running:
+                yield self.db.stopService()
 
     @defer.inlineCallbacks
     def reconfig(self):
@@ -389,7 +394,7 @@ class BuildMaster(service.ReconfigurableServiceMixin, service.MasterService):
         # longer on larger installations and may take a while.
         self.reconfig_notifier = task.LoopingCall(
             lambda: log.msg(
-                "reconfig is ongoing for " f"{self.reactor.seconds() - self.reconfig_active:.3f} s"
+                f"reconfig is ongoing for {self.reactor.seconds() - self.reconfig_active:.3f} s"
             )
         )
         self.reconfig_notifier.start(10, now=False)
@@ -433,8 +438,8 @@ class BuildMaster(service.ReconfigurableServiceMixin, service.MasterService):
                 log.msg(msg)
             failed = True
 
-        except Exception:
-            log.err(failure.Failure(), 'during reconfig:')
+        except Exception as e:
+            log.err(e, 'during reconfig:')
             failed = True
 
         finally:
@@ -450,20 +455,16 @@ class BuildMaster(service.ReconfigurableServiceMixin, service.MasterService):
 
         log.msg(f"{msg} (took {(self.reactor.seconds() - time_started):.3f} seconds)")
 
+    @defer.inlineCallbacks
     def reconfigServiceWithBuildbotConfig(self, new_config):
-        if self.configured_db_url is None:
-            self.configured_db_url = new_config.db['db_url']
-        elif self.configured_db_url != new_config.db['db_url']:
-            config.error(
-                "Cannot change c['db']['db_url'] after the master has started",
-            )
-
         if self.config.mq['type'] != new_config.mq['type']:
             raise config.ConfigErrors([
                 "Cannot change c['mq']['type'] after the master has started",
             ])
 
-        return super().reconfigServiceWithBuildbotConfig(new_config)
+        yield super().reconfigServiceWithBuildbotConfig(new_config)
+        # db must come later so that it has access to newly configured services
+        yield self.db.reconfigServiceWithBuildbotConfig(new_config)
 
     # informational methods
     def allSchedulers(self):
