@@ -13,8 +13,12 @@
 #
 # Copyright Buildbot Team Members
 
+from __future__ import annotations
 
 from contextlib import contextmanager
+from typing import TYPE_CHECKING
+from typing import ClassVar
+from typing import Sequence
 
 from twisted.python import deprecate
 from twisted.python import versions
@@ -33,6 +37,9 @@ from buildbot.steps.shell import Test
 from buildbot.steps.source.cvs import CVS
 from buildbot.steps.source.svn import SVN
 
+if TYPE_CHECKING:
+    from buildbot.process.builder import Builder
+
 
 # deprecated, use BuildFactory.addStep
 @deprecate.deprecated(versions.Version("buildbot", 0, 8, 6))
@@ -48,23 +55,23 @@ class BuildFactory(util.ComparableMixin):
     @type  buildClass: L{buildbot.process.build.Build}
     """
 
-    buildClass = Build
-    useProgress = 1
+    buildClass: type[Build] = Build
+    useProgress = True
     workdir = "build"
-    compare_attrs = ('buildClass', 'steps', 'useProgress', 'workdir')
+    compare_attrs: ClassVar[Sequence[str]] = ('buildClass', 'steps', 'useProgress', 'workdir')
 
     def __init__(self, steps=None):
         self.steps = []
         if steps:
             self.addSteps(steps)
 
-    def newBuild(self, requests):
+    def newBuild(self, requests, builder: Builder) -> Build:
         """Create a new Build instance.
 
         @param requests: a list of buildrequest dictionaries describing what is
         to be built
         """
-        b = self.buildClass(requests)
+        b = self.buildClass(requests, builder)
         b.useProgress = self.useProgress
         b.workdir = self.workdir
         b.setStepFactories(self.steps)
@@ -139,7 +146,7 @@ class GNUAutoconf(BuildFactory):
             if isinstance(configure, str):
                 if configureFlags:
                     assert " " not in configure  # please use list instead
-                    command = [configure] + configureFlags
+                    command = [configure, *configureFlags]
                 else:
                     command = configure
             else:
@@ -215,7 +222,7 @@ class Trial(BuildFactory):
 
         from buildbot.steps.python_twisted import Trial
 
-        buildcommand = buildpython + ["./setup.py", "build"]
+        buildcommand = [*buildpython, "./setup.py", "build"]
         self.addStep(Compile(command=buildcommand, env=env))
         self.addStep(
             Trial(
