@@ -12,6 +12,10 @@
 # Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 # Copyright Buildbot Team Members
+from __future__ import annotations
+
+from collections import defaultdict
+from typing import TYPE_CHECKING
 
 from twisted.internet import defer
 from twisted.internet import error
@@ -25,10 +29,21 @@ from buildbot.process import metrics
 from buildbot.process.results import CANCELLED
 from buildbot.process.results import FAILURE
 from buildbot.process.results import SUCCESS
-from buildbot.util import lineboundaries
 from buildbot.util.eventual import eventually
+from buildbot.util.lineboundaries import LineBoundaryFinder
+from buildbot.util.twisted import async_to_deferred
 from buildbot.worker.protocols import base
 
+if TYPE_CHECKING:
+    from typing import Any
+    from typing import Awaitable
+    from typing import Callable
+
+    from buildbot.process.buildstep import BuildStep
+    from buildbot.process.log import StreamLog
+    from buildbot.worker.base import AbstractWorker
+    from buildbot.worker.protocols.base import Connection
+
 
 class RemoteException(Exception):
     pass
@@ -39,66 +54,80 @@ class RemoteCommand(base.RemoteCommandImpl):
     _commandCounter = 0
 
     active = False
-    rc = None
+    rc: int | None = None
     debug = False
 
     def __init__(
         self,
-        remote_command,
-        args,
-        ignore_updates=False,
-        collectStdout=False,
-        collectStderr=False,
-        decodeRC=None,
-        stdioLogName='stdio',
-    ):
+        remote_command: str,
+        args: dict[str, Any],
+        ignore_updates: bool = False,
+        collectStdout: bool = False,
+        collectStderr: bool = False,
+        decodeRC: dict[int, int] | None = None,
+        stdioLogName: str = 'stdio',
+    ) -> None:
         if decodeRC is None:
             decodeRC = {0: SUCCESS}
-        self.logs = {}
-        self.delayedLogs = {}
-        self._closeWhenFinished = {}
-        self.collectStdout = collectStdout
-        self.collectStderr = collectStderr
+        self.logs: dict[str, StreamLog] = {}
+        self.delayedLogs: dict[
+            str,
+            tuple[
+                Callable[[RemoteCommand], Awaitable[StreamLog]],
+                bool,
+            ],
+        ] = {}
+        self._closeWhenFinished: dict[str, bool] = {}
+        self.collectStdout: bool = collectStdout
+        self.collectStderr: bool = collectStderr
         self.stdout = ''
         self.stderr = ''
-        self.updates = {}
-        self.stdioLogName = stdioLogName
-        self._startTime = None
-        self._remoteElapsed = None
+        self.updates: defaultdict[str, list[Any]] = defaultdict(list)
+        self.stdioLogName: str = stdioLogName
+        self._startTime: float | None = None
+        self._remoteElapsed: float | None = None
         self.remote_failure_reason = None
-        self.remote_command = remote_command
-        self.args = args
-        self.ignore_updates = ignore_updates
-        self.decodeRC = decodeRC
-        self.conn = None
+        self.remote_command: str = remote_command
+        self.args: dict[str, Any] = args
+        self.ignore_updates: bool = ignore_updates
+        self.decodeRC: dict[int, int] = decodeRC
+        self.conn: Connection | None = None
         self._is_conn_test_fake = False
-        self.worker = None
-        self.step = None
-        self.builder_name = None
-        self.commandID = None
-        self.deferred = None
-        self.interrupted = False
+        self.worker: AbstractWorker | None = None
+        self.step: BuildStep | None = None
+        self.builder_name: str | None = None
+        self.commandID: str | None = None
+        self.deferred: defer.Deferred[RemoteCommand] | None = None
+        self.interrupted: bool = False
         # a lock to make sure that only one log-handling method runs at a time.
         # This is really only a problem with old-style steps, which do not
         # wait for the Deferred from one method before invoking the next.
         self.loglock = defer.DeferredLock()
-        self._line_boundary_finders = {}
+        self._line_boundary_finders: defaultdict[
+            str,
+            LineBoundaryFinder,
+        ] = defaultdict(LineBoundaryFinder)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return f"<RemoteCommand '{self.remote_command}' at {id(self)}>"
 
     @classmethod
-    def generate_new_command_id(cls):
+    def generate_new_command_id(cls) -> str:
         cmd_id = cls._commandCounter
         cls._commandCounter += 1
         return f"{cmd_id}"
 
     @classmethod
-    def get_last_generated_command_id(cls):
+    def get_last_generated_command_id(cls) -> str:
         cmd_id = cls._commandCounter - 1
         return f"{cmd_id}"
 
-    def run(self, step, conn, builder_name):
+    def run(
+        self,
+        step: BuildStep,
+        conn: Connection,
+        builder_name: str,
+    ) -> defer.Deferred[RemoteCommand]:
         self.active = True
         self.step = step
         self.conn = conn
@@ -126,7 +155,12 @@ class RemoteCommand(base.RemoteCommandImpl):
         # when our parent Step calls our .lostRemote() method.
         return self.deferred
 
-    def useLog(self, log_, closeWhenFinished=False, logfileName=None):
+    def useLog(
+        self,
+        log_: StreamLog,
+        closeWhenFinished: bool = False,
+        logfileName: str | None = None,
+    ) -> None:
         # NOTE: log may be a SyngLogFileWrapper or a Log instance, depending on
         # the step
         if not logfileName:
@@ -136,24 +170,30 @@ class RemoteCommand(base.RemoteCommandImpl):
         self.logs[logfileName] = log_
         self._closeWhenFinished[logfileName] = closeWhenFinished
 
-    def useLogDelayed(self, logfileName, activateCallBack, closeWhenFinished=False):
+    def useLogDelayed(
+        self,
+        logfileName: str,
+        activateCallBack: Callable[[RemoteCommand], Awaitable[StreamLog]],
+        closeWhenFinished: bool = False,
+    ) -> None:
         assert logfileName not in self.logs
         assert logfileName not in self.delayedLogs
         self.delayedLogs[logfileName] = (activateCallBack, closeWhenFinished)
 
-    def _start(self):
+    def _start(self) -> defer.Deferred:
         self._startTime = util.now()
         # This method only initiates the remote command.
         # We will receive remote_update messages as the command runs.
         # We will get a single remote_complete when it finishes.
         # We should fire self.deferred when the command is done.
+        assert self.conn is not None
         d = self.conn.remoteStartCommand(
             self, self.builder_name, self.commandID, self.remote_command, self.args
         )
         return d
 
-    @defer.inlineCallbacks
-    def _finished(self, failure=None):
+    @async_to_deferred
+    async def _finished(self, failure=None) -> None:
         # Finished may be called concurrently by a message from worker and interruption due to
         # lost connection.
         if not self.active:
@@ -166,18 +206,19 @@ class RemoteCommand(base.RemoteCommandImpl):
         if not self._is_conn_test_fake:
             timeout = 10
             while self.rc is None and timeout > 0:
-                yield util.asyncSleep(0.1)
+                await util.asyncSleep(0.1)
                 timeout -= 1
 
+        assert self.deferred is not None
         try:
-            yield self.remoteComplete(failure)
+            await self.remoteComplete(failure)
             # this fires the original deferred we returned from .run(),
             self.deferred.callback(self)
         except Exception as e:
             self.deferred.errback(e)
 
-    @defer.inlineCallbacks
-    def interrupt(self, why):
+    @async_to_deferred
+    async def interrupt(self, why) -> None:
         log.msg("RemoteCommand.interrupt", self, why)
 
         if self.conn and isinstance(why, Failure) and why.check(error.ConnectionLost):
@@ -199,12 +240,13 @@ class RemoteCommand(base.RemoteCommandImpl):
         # when the interrupt command has been delivered.
 
         try:
-            yield self.conn.remoteInterruptCommand(self.builder_name, self.commandID, str(why))
+            await self.conn.remoteInterruptCommand(self.builder_name, self.commandID, str(why))
             # the worker may not have remote_interruptCommand
         except Exception as e:
             log.msg("RemoteCommand.interrupt failed", self, e)
 
-    def remote_update_msgpack(self, updates):
+    def remote_update_msgpack(self, updates: list[tuple[str, Any]]) -> None:
+        assert self.worker is not None
         self.worker.messageReceivedFromWorker()
         try:
             for key, value in updates:
@@ -220,14 +262,10 @@ class RemoteCommand(base.RemoteCommandImpl):
             # log failure, terminate build, let worker retire the update
             self._finished(Failure())
 
-    def split_line(self, stream, text):
-        try:
-            return self._line_boundary_finders[stream].append(text)
-        except KeyError:
-            lbf = self._line_boundary_finders[stream] = lineboundaries.LineBoundaryFinder()
-            return lbf.append(text)
+    def split_line(self, stream: str, text: str) -> str | None:
+        return self._line_boundary_finders[stream].append(text)
 
-    def remote_update(self, updates):
+    def remote_update(self, updates: list[tuple[dict[str | bytes, Any], int]]) -> int:
         """
         I am called by the worker's
         L{buildbot_worker.base.WorkerForBuilderBase.sendUpdate} so
@@ -236,23 +274,27 @@ class RemoteCommand(base.RemoteCommandImpl):
         @type  updates: list of [object, int]
         @param updates: list of updates from the remote command
         """
-        updates = decode(updates)
+        assert self.worker is not None
         self.worker.messageReceivedFromWorker()
         max_updatenum = 0
-        for update, num in updates:  # noqa pylint: disable=too-many-nested-blocks
-            # log.msg("update[%d]:" % num)
+        for update, num in updates:
             try:
                 if self.active and not self.ignore_updates:
                     for key, value in update.items():
+                        key = util.bytes2unicode(key)
+                        value = decode(value)
                         if key in ['stdout', 'stderr', 'header']:
+                            assert isinstance(value, str), type(value)
                             whole_line = self.split_line(key, value)
                             if whole_line is not None:
                                 self.remoteUpdate(key, whole_line, False)
                         elif key == "log":
                             logname, data = value
+                            assert isinstance(logname, str), type(logname)
+                            assert isinstance(data, str), type(data)
                             whole_line = self.split_line(logname, data)
-                            value = (logname, whole_line)
                             if whole_line is not None:
+                                value = (logname, whole_line)
                                 self.remoteUpdate(key, value, False)
                         else:
                             self.remoteUpdate(key, value, False)
@@ -265,7 +307,7 @@ class RemoteCommand(base.RemoteCommandImpl):
             max_updatenum = max(max_updatenum, num)
         return max_updatenum
 
-    def remote_complete(self, failure=None):
+    def remote_complete(self, failure=None) -> None:
         """
         Called by the worker's
         L{buildbot_worker.base.WorkerForBuilderBase.commandComplete} to
@@ -275,100 +317,99 @@ class RemoteCommand(base.RemoteCommandImpl):
 
         @rtype: None
         """
+        assert self.worker is not None
         self.worker.messageReceivedFromWorker()
         # call the real remoteComplete a moment later, but first return an
         # acknowledgement so the worker can retire the completion message.
         if self.active:
             eventually(self._finished, failure)
-        return None
 
     @util.deferredLocked('loglock')
-    def addStdout(self, data):
+    @async_to_deferred
+    async def addStdout(self, data) -> None:
         if self.collectStdout:
             self.stdout += data
         if self.stdioLogName is not None and self.stdioLogName in self.logs:
-            self.logs[self.stdioLogName].addStdout(data)
-        return defer.succeed(None)
+            await self.logs[self.stdioLogName].addStdout(data)
 
     @util.deferredLocked('loglock')
-    def add_stdout_lines(self, data, is_flushed):
+    @async_to_deferred
+    async def add_stdout_lines(self, data, is_flushed: bool) -> None:
         if self.collectStdout:
             if is_flushed:
                 data = data[:-1]
             self.stdout += data
         if self.stdioLogName is not None and self.stdioLogName in self.logs:
-            self.logs[self.stdioLogName].add_stdout_lines(data)
-        return defer.succeed(None)
+            await self.logs[self.stdioLogName].add_stdout_lines(data)
 
     @util.deferredLocked('loglock')
-    def addStderr(self, data):
+    @async_to_deferred
+    async def addStderr(self, data) -> None:
         if self.collectStderr:
             self.stderr += data
         if self.stdioLogName is not None and self.stdioLogName in self.logs:
-            self.logs[self.stdioLogName].addStderr(data)
-        return defer.succeed(None)
+            await self.logs[self.stdioLogName].addStderr(data)
 
     @util.deferredLocked('loglock')
-    def add_stderr_lines(self, data, is_flushed):
+    @async_to_deferred
+    async def add_stderr_lines(self, data, is_flushed) -> None:
         if self.collectStderr:
             if is_flushed:
                 data = data[:-1]
             self.stderr += data
         if self.stdioLogName is not None and self.stdioLogName in self.logs:
-            self.logs[self.stdioLogName].add_stderr_lines(data)
-        return defer.succeed(None)
+            await self.logs[self.stdioLogName].add_stderr_lines(data)
 
     @util.deferredLocked('loglock')
-    def addHeader(self, data):
+    @async_to_deferred
+    async def addHeader(self, data) -> None:
         if self.stdioLogName is not None and self.stdioLogName in self.logs:
-            self.logs[self.stdioLogName].addHeader(data)
-        return defer.succeed(None)
+            await self.logs[self.stdioLogName].addHeader(data)
 
     @util.deferredLocked('loglock')
-    def add_header_lines(self, data):
+    @async_to_deferred
+    async def add_header_lines(self, data) -> None:
         if self.stdioLogName is not None and self.stdioLogName in self.logs:
-            self.logs[self.stdioLogName].add_header_lines(data)
-        return defer.succeed(None)
+            await self.logs[self.stdioLogName].add_header_lines(data)
 
     @util.deferredLocked('loglock')
-    @defer.inlineCallbacks
-    def addToLog(self, logname, data):
+    @async_to_deferred
+    async def addToLog(self, logname, data) -> None:
         # Activate delayed logs on first data.
         if logname in self.delayedLogs:
-            (activateCallBack, closeWhenFinished) = self.delayedLogs[logname]
-            del self.delayedLogs[logname]
-            loog = yield activateCallBack(self)
+            (activateCallBack, closeWhenFinished) = self.delayedLogs.pop(logname)
+            loog = await activateCallBack(self)
             self.logs[logname] = loog
             self._closeWhenFinished[logname] = closeWhenFinished
 
         if logname in self.logs:
-            yield self.logs[logname].add_stdout_lines(data)
+            await self.logs[logname].add_stdout_lines(data)
         else:
             log.msg(f"{self}.addToLog: no such log {logname}")
 
     @metrics.countMethod('RemoteCommand.remoteUpdate()')
-    @defer.inlineCallbacks
-    def remoteUpdate(self, key, value, is_flushed):
+    @async_to_deferred
+    async def remoteUpdate(self, key: str, value, is_flushed: bool) -> None:
         def cleanup(data):
-            if self.step is None:
+            if self.step is None or self.step.build is None:
                 return data
             return self.step.build.properties.cleanupTextFromSecrets(data)
 
         if self.debug:
             log.msg(f"Update[{key}]: {value}")
         if key == "stdout":
-            yield self.add_stdout_lines(cleanup(value), is_flushed)
+            await self.add_stdout_lines(cleanup(value), is_flushed)
         if key == "stderr":
-            yield self.add_stderr_lines(cleanup(value), is_flushed)
+            await self.add_stderr_lines(cleanup(value), is_flushed)
         if key == "header":
-            yield self.add_header_lines(cleanup(value))
+            await self.add_header_lines(cleanup(value))
         if key == "log":
             logname, data = value
-            yield self.addToLog(logname, cleanup(data))
+            await self.addToLog(logname, cleanup(data))
         if key == "rc":
             rc = self.rc = value
             log.msg(f"{self} rc={rc}")
-            yield self.add_header_lines(f"program finished with exit code {rc}\n")
+            await self.add_header_lines(f"program finished with exit code {rc}\n")
         if key == "elapsed":
             self._remoteElapsed = value
         if key == "failure_reason":
@@ -376,12 +417,10 @@ class RemoteCommand(base.RemoteCommandImpl):
 
         # TODO: these should be handled at the RemoteCommand level
         if key not in ('stdout', 'stderr', 'header', 'rc', "failure_reason"):
-            if key not in self.updates:
-                self.updates[key] = []
             self.updates[key].append(value)
 
-    @defer.inlineCallbacks
-    def remoteComplete(self, maybeFailure):
+    @async_to_deferred
+    async def remoteComplete(self, maybeFailure) -> None:
         if self._startTime and self._remoteElapsed:
             delta = (util.now() - self._startTime) - self._remoteElapsed
             metrics.MetricTimeEvent.log("RemoteCommand.overhead", delta)
@@ -390,25 +429,22 @@ class RemoteCommand(base.RemoteCommandImpl):
             if key in ['stdout', 'stderr', 'header']:
                 whole_line = lbf.flush()
                 if whole_line is not None:
-                    yield self.remoteUpdate(key, whole_line, True)
+                    await self.remoteUpdate(key, whole_line, True)
             else:
                 logname = key
                 whole_line = lbf.flush()
                 value = (logname, whole_line)
                 if whole_line is not None:
-                    yield self.remoteUpdate("log", value, True)
+                    await self.remoteUpdate("log", value, True)
 
-        try:
-            yield self.loglock.acquire()
+        async with self.loglock:
             for name, loog in self.logs.items():
                 if self._closeWhenFinished[name]:
                     if maybeFailure:
-                        yield loog.addHeader(f"\nremoteFailed: {maybeFailure}")
+                        await loog.addHeader(f"\nremoteFailed: {maybeFailure}")
                     else:
                         log.msg(f"closing log {loog}")
-                    yield loog.finish()
-        finally:
-            yield self.loglock.release()
+                    await loog.finish()
 
         if maybeFailure:
             # Message Pack protocol can not send an exception object back to the master, so
@@ -425,14 +461,14 @@ class RemoteCommand(base.RemoteCommandImpl):
                 maybeFailure.type = RemoteException
             maybeFailure.raiseException()
 
-    def results(self):
+    def results(self) -> int:
         if self.interrupted:
             return CANCELLED
         if self.rc in self.decodeRC:
             return self.decodeRC[self.rc]
         return FAILURE
 
-    def didFail(self):
+    def didFail(self) -> bool:
         return self.results() == FAILURE
 
 
@@ -449,6 +485,7 @@ class RemoteShellCommand(RemoteCommand):
         want_stderr=1,
         timeout=20 * 60,
         maxTime=None,
+        max_lines=None,
         sigtermTime=None,
         logfiles=None,
         usePTY=None,
@@ -491,6 +528,7 @@ class RemoteShellCommand(RemoteCommand):
             'logfiles': logfiles,
             'timeout': timeout,
             'maxTime': maxTime,
+            'max_lines': max_lines,
             'sigtermTime': sigtermTime,
             'usePTY': usePTY,
             'logEnviron': logEnviron,
@@ -530,4 +568,4 @@ class RemoteShellCommand(RemoteCommand):
         return super()._start()
 
     def __repr__(self):
-        return f"<RemoteShellCommand '{repr(self.fake_command)}'>"
+        return f"<RemoteShellCommand '{self.fake_command!r}'>"
