@@ -11,7 +11,9 @@
 # this program; if not, write to the Free Software Foundation, Inc., 51
 # Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
-# Copyright Buildbot Team Members
+# Copyright Buildbot Team Member
+
+from __future__ import annotations
 
 from urllib.parse import quote_plus as urlquote_plus
 
@@ -38,7 +40,7 @@ HOSTED_BASE_URL = 'https://gitlab.com'
 
 
 class GitLabStatusPush(ReporterBase):
-    name = "GitLabStatusPush"
+    name: str | None = "GitLabStatusPush"  # type: ignore[assignment]
 
     def checkConfig(
         self,
@@ -55,7 +57,6 @@ class GitLabStatusPush(ReporterBase):
             generators = self._create_default_generators()
 
         super().checkConfig(generators=generators, **kwargs)
-        httpclientservice.HTTPClientService.checkAvailable(self.__class__.__name__)
 
     @defer.inlineCallbacks
     def reconfigService(
@@ -85,8 +86,8 @@ class GitLabStatusPush(ReporterBase):
         if baseURL.endswith('/'):
             baseURL = baseURL[:-1]
         self.baseURL = baseURL
-        self._http = yield httpclientservice.HTTPClientService.getService(
-            self.master,
+        self._http = yield httpclientservice.HTTPSession(
+            self.master.httpservice,
             baseURL,
             headers={'PRIVATE-TOKEN': token},
             debug=self.debug,
@@ -149,8 +150,7 @@ class GitLabStatusPush(ReporterBase):
             proj = yield response.json()
             if response.code not in (200,):
                 log.msg(
-                    'Unknown (or hidden) gitlab project'
-                    f'{project_full_name}: {proj.get("message")}'
+                    f'Unknown (or hidden) gitlab project{project_full_name}: {proj.get("message")}'
                 )
                 return None
             self.project_ids[project_full_name] = proj['id']
@@ -222,8 +222,5 @@ class GitLabStatusPush(ReporterBase):
             except Exception as e:
                 log.err(
                     e,
-                    (
-                        f'Failed to send status "{state}" for '
-                        f'{sourcestamp["repository"]} at {sha}'
-                    ),
+                    (f'Failed to send status "{state}" for {sourcestamp["repository"]} at {sha}'),
                 )
