@@ -13,6 +13,8 @@
 #
 # Copyright Buildbot Team Members
 
+from __future__ import annotations
+
 from twisted.internet import defer
 from twisted.python import log
 
@@ -23,7 +25,7 @@ from buildbot.util import httpclientservice
 
 
 class HttpStatusPush(ReporterBase):
-    name = "HttpStatusPush"
+    name: str | None = "HttpStatusPush"  # type: ignore[assignment]
     secrets = ["auth"]
 
     def checkConfig(
@@ -33,7 +35,6 @@ class HttpStatusPush(ReporterBase):
             generators = self._create_default_generators()
 
         super().checkConfig(generators=generators, **kwargs)
-        httpclientservice.HTTPClientService.checkAvailable(self.__class__.__name__)
 
     @defer.inlineCallbacks
     def reconfigService(
@@ -47,8 +48,13 @@ class HttpStatusPush(ReporterBase):
 
         yield super().reconfigService(generators=generators, **kwargs)
 
-        self._http = yield httpclientservice.HTTPClientService.getService(
-            self.master, serverUrl, auth=auth, headers=headers, debug=self.debug, verify=self.verify
+        self._http = yield httpclientservice.HTTPSession(
+            self.master.httpservice,
+            serverUrl,
+            auth=auth,
+            headers=headers,
+            debug=self.debug,
+            verify=self.verify,
         )
 
     def _create_default_generators(self):
