@@ -13,10 +13,15 @@
 #
 # Copyright Buildbot Team Members
 
+from __future__ import annotations
+
 import io
 import json
 import random
 import shlex
+from typing import Any
+from typing import ClassVar
+from typing import Sequence
 
 from twisted.internet import defer
 from twisted.internet import reactor
@@ -152,7 +157,7 @@ class TelegramContact(Contact):
         else:
             self.send(f"Never mind, {self.user_name}...")
 
-    command_NAY.usage = "nay - never mind the command we are currently discussing"
+    command_NAY.usage = "nay - never mind the command we are currently discussing"  # type: ignore[attr-defined]
 
     @classmethod
     def get_commands(cls):
@@ -191,7 +196,7 @@ class TelegramContact(Contact):
             yield self.send(f"{self.user_name}, your ID is `{self.user_id}`.")
             self.send(f'This {self.channel.chat_info.get("type", "group")} ID is `{self.chat_id}`.')
 
-    command_GETID.usage = (
+    command_GETID.usage = (  # type: ignore[attr-defined]
         "getid - get user and chat ID that can be put in the master configuration file"
     )
 
@@ -465,7 +470,7 @@ class TelegramContact(Contact):
                 sched = next(iter(forceschedulers))
             else:
                 raise UsageError(
-                    "Try '/force' and follow the instructions" f" (no force scheduler {sched})"
+                    f"Try '/force' and follow the instructions (no force scheduler {sched})"
                 )
         scheduler = forceschedulers[sched]
 
@@ -605,7 +610,7 @@ class TelegramContact(Contact):
         else:
             raise UsageError("Try '/force' and follow the instructions")
 
-    command_FORCE.usage = "force - Force a build"
+    command_FORCE.usage = "force - Force a build"  # type: ignore[attr-defined]
 
 
 class TelegramStatusBot(StatusBot):
@@ -617,7 +622,7 @@ class TelegramStatusBot(StatusBot):
     idle_string = "idle 💤"
     running_string = "running 🌀:"
 
-    query_cache = {}
+    query_cache: dict[int, dict[str, Any]] = {}
 
     @property
     def commandSuffix(self):
@@ -878,7 +883,7 @@ class TelegramStatusBot(StatusBot):
 
 
 class TelegramWebhookBot(TelegramStatusBot):
-    name = "TelegramWebhookBot"
+    name: str | None = "TelegramWebhookBot"  # type: ignore[assignment]
 
     def __init__(self, token, *args, certificate=None, **kwargs):
         TelegramStatusBot.__init__(self, token, *args, **kwargs)
@@ -921,7 +926,7 @@ class TelegramWebhookBot(TelegramStatusBot):
 
 
 class TelegramPollingBot(TelegramStatusBot):
-    name = "TelegramPollingBot"
+    name: str | None = "TelegramPollingBot"  # type: ignore[assignment]
 
     def __init__(self, *args, poll_timeout=120, **kwargs):
         super().__init__(*args, **kwargs)
@@ -979,7 +984,7 @@ class TelegramBot(service.BuildbotService):
 
     in_test_harness = False
 
-    compare_attrs = [
+    compare_attrs: ClassVar[Sequence[str]] = [
         "bot_token",
         "chat_ids",
         "authz",
@@ -1000,7 +1005,7 @@ class TelegramBot(service.BuildbotService):
 
     def _get_http(self, bot_token):
         base_url = "https://api.telegram.org/bot" + bot_token
-        return httpclientservice.HTTPClientService.getService(self.master, base_url)
+        return httpclientservice.HTTPSession(self.master.httpservice, base_url)
 
     def checkConfig(
         self,
