@@ -43,12 +43,11 @@ class ReconfigError(Exception):
 
 
 class TailProcess(protocol.ProcessProtocol):
-
     def outReceived(self, data):
         self.lw.dataReceived(data)
 
     def errReceived(self, data):
-        self.lw.print_output("ERR: '{}'".format(data))
+        self.lw.print_output(f"ERR: '{data}'")
 
 
 class LineOnlyLongLineReceiver(protocol.Protocol):
@@ -56,6 +55,7 @@ class LineOnlyLongLineReceiver(protocol.Protocol):
     This is almost the same as Twisted's LineOnlyReceiver except that long lines are handled
     appropriately.
     """
+
     _buffer = b''
     delimiter = b'\r\n'
     MAX_LENGTH = 16384
@@ -114,8 +114,7 @@ class LogWatcher(LineOnlyLongLineReceiver):
             tailBin = "/usr/bin/tail"
 
         args = ("tail", "-f", "-n", "0", self.logfile)
-        self.p = self._reactor.spawnProcess(self.pp, tailBin, args,
-                                            env=os.environ)
+        self.p = self._reactor.spawnProcess(self.pp, tailBin, args, env=os.environ)
         self.running = True
         d = defer.maybeDeferred(self._start)
         return d
@@ -152,13 +151,14 @@ class LogWatcher(LineOnlyLongLineReceiver):
 
     def create_logfile(self, path):  # pragma: no cover
         if not os.path.exists(path):
-            open(path, 'a').close()
+            with open(path, 'a', encoding='utf-8'):
+                pass
 
     def print_output(self, output):  # pragma: no cover
         print(output)
 
     def lineLengthExceeded(self, line):
-        msg = 'Got an a very long line in the log (length {} bytes), ignoring'.format(len(line))
+        msg = f'Got an a very long line in the log (length {len(line)} bytes), ignoring'
         self.print_output(msg)
 
     def lineReceived(self, line):
@@ -174,8 +174,14 @@ class LogWatcher(LineOnlyLongLineReceiver):
 
         # certain lines indicate progress, so we "cancel" the timeout
         # and it will get re-added when it fires
-        PROGRESS_TEXT = [b'Starting BuildMaster', b'Loading configuration from', b'added builder',
-                         b'adding scheduler', b'Loading builder', b'Starting factory']
+        PROGRESS_TEXT = [
+            b'Starting BuildMaster',
+            b'Loading configuration from',
+            b'added builder',
+            b'adding scheduler',
+            b'Loading builder',
+            b'Starting factory',
+        ]
         for progressText in PROGRESS_TEXT:
             if progressText in line:
                 self.timer = None
@@ -183,7 +189,10 @@ class LogWatcher(LineOnlyLongLineReceiver):
 
         if b"message from master: attached" in line:
             return self.finished("worker")
-        if b"reconfig aborted" in line or b'reconfig partially applied' in line:
+        if (
+            b"configuration update aborted" in line
+            or b'configuration update partially applied' in line
+        ):
             return self.finished(Failure(ReconfigError()))
         if b"Server Shut Down" in line:
             return self.finished(Failure(ReconfigError()))
