@@ -16,8 +16,11 @@
 manage providers and handle secrets
 """
 
+from __future__ import annotations
+
 from twisted.internet import defer
 
+from buildbot.secrets.providers.base import SecretProviderBase
 from buildbot.secrets.secret import SecretDetails
 from buildbot.util import service
 
@@ -27,9 +30,18 @@ class SecretManager(service.BuildbotServiceManager):
     Secret manager
     """
 
-    name = 'secrets'
+    name: str | None = 'secrets'  # type: ignore[assignment]
     config_attr = "secretsProviders"
 
+    @defer.inlineCallbacks
+    def setup(self):
+        configuredProviders = self.get_service_config(self.master.config)
+
+        for child in configuredProviders.values():
+            assert isinstance(child, SecretProviderBase)
+            yield child.setServiceParent(self)
+            yield child.configureService()
+
     @defer.inlineCallbacks
     def get(self, secret, *args, **kwargs):
         """
