@@ -16,6 +16,8 @@
 from __future__ import annotations
 
 import hashlib
+from typing import ClassVar
+from typing import Sequence
 
 from twisted.application import service
 from twisted.internet import defer
@@ -26,6 +28,7 @@ from twisted.python.reflect import accumulateClassList
 
 import buildbot.config
 from buildbot import util
+from buildbot.process.properties import Properties
 from buildbot.util import bytes2unicode
 from buildbot.util import config
 from buildbot.util import unicode2bytes
@@ -54,6 +57,8 @@ class ReconfigurableServiceMixin:
 # twisted 16's Service is now an new style class, better put everybody new style
 # to catch issues even on twisted < 16
 class AsyncService(service.Service):
+    name: str | None  # type: ignore[assignment]
+
     # service.Service.setServiceParent does not wait for neither disownServiceParent nor addService
     # to complete
     @defer.inlineCallbacks
@@ -116,7 +121,7 @@ class AsyncMultiService(AsyncService, service.MultiService):
     def addService(self, service):
         if service.name is not None:
             if service.name in self.namedServices:
-                raise RuntimeError("cannot have two services with same name" f" '{service.name}'")
+                raise RuntimeError(f"cannot have two services with same name '{service.name}'")
             self.namedServices[service.name] = service
         self.services.append(service)
         if self.running:
@@ -133,6 +138,11 @@ class MasterService(AsyncMultiService):
     def master(self):
         return self
 
+    def get_db_url(self, new_config) -> defer.Deferred:
+        p = Properties()
+        p.master = self
+        return p.render(new_config.db['db_url'])
+
 
 class SharedService(AsyncMultiService):
     """a service that is created only once per parameter set in a parent service"""
@@ -177,10 +187,10 @@ class SharedService(AsyncMultiService):
 class BuildbotService(
     AsyncMultiService, config.ConfiguredMixin, util.ComparableMixin, ReconfigurableServiceMixin
 ):
-    compare_attrs = ('name', '_config_args', '_config_kwargs')
-    name = None
+    compare_attrs: ClassVar[Sequence[str]] = ('name', '_config_args', '_config_kwargs')
+    name: str | None = None  # type: ignore[assignment]
     configured = False
-    objectid = None
+    objectid: int | None = None
 
     def __init__(self, *args, **kwargs):
         name = kwargs.pop("name", None)
@@ -212,8 +222,6 @@ class BuildbotService(
             return None
         self.configured = True
         # render renderables in parallel
-        # Properties import to resolve cyclic import issue
-        from buildbot.process.properties import Properties
 
         p = Properties()
         p.master = self.master
@@ -255,9 +263,6 @@ class BuildbotService(
         return defer.succeed(None)
 
     def renderSecrets(self, *args):
-        # Properties import to resolve cyclic import issue
-        from buildbot.process.properties import Properties
-
         p = Properties()
         p.master = self.master
 
@@ -279,11 +284,11 @@ class ClusteredBuildbotService(BuildbotService):
     - return after it starts else.
     """
 
-    compare_attrs = ('name',)
+    compare_attrs: ClassVar[Sequence[str]] = ('name',)
 
     POLL_INTERVAL_SEC = 5 * 60  # 5 minutes
 
-    serviceid = None
+    serviceid: int | None = None
     active = False
 
     def __init__(self, *args, **kwargs):
@@ -437,7 +442,7 @@ class ClusteredBuildbotService(BuildbotService):
 
 class BuildbotServiceManager(AsyncMultiService, config.ConfiguredMixin, ReconfigurableServiceMixin):
     config_attr = "services"
-    name = "services"
+    name: str | None = "services"  # type: ignore[assignment]
 
     def getConfigDict(self):
         return {
@@ -488,8 +493,7 @@ class BuildbotServiceManager(AsyncMultiService, config.ConfiguredMixin, Reconfig
 
         if removed_names or added_names:
             log.msg(
-                f"adding {len(added_names)} new {self.config_attr}, "
-                f"removing {len(removed_names)}"
+                f"adding {len(added_names)} new {self.config_attr}, removing {len(removed_names)}"
             )
 
             for n in removed_names:
