@@ -53,7 +53,7 @@ class GitHubEventHandler(PullRequestMixin):
         pullrequest_ref=None,
         token=None,
         debug=False,
-        verify=False,
+        verify=True,
     ):
         if github_property_whitelist is None:
             github_property_whitelist = []
@@ -237,12 +237,13 @@ class GitHubEventHandler(PullRequestMixin):
         if self._token:
             p = Properties()
             p.master = self.master
+            p.setProperty("full_name", repo, "change_hook")
             token = yield p.render(self._token)
             headers['Authorization'] = 'token ' + token
 
         url = f'/repos/{repo}/commits/{sha}'
-        http = yield httpclientservice.HTTPClientService.getService(
-            self.master,
+        http = yield httpclientservice.HTTPSession(
+            self.master.httpservice,
             self.github_api_endpoint,
             headers=headers,
             debug=self.debug,
@@ -268,12 +269,13 @@ class GitHubEventHandler(PullRequestMixin):
         if self._token:
             p = Properties()
             p.master = self.master
+            p.setProperty("full_name", repo, "change_hook")
             token = yield p.render(self._token)
             headers["Authorization"] = "token " + token
 
         url = f"/repos/{repo}/pulls/{number}/files"
-        http = yield httpclientservice.HTTPClientService.getService(
-            self.master,
+        http = yield httpclientservice.HTTPSession(
+            self.master.httpservice,
             self.github_api_endpoint,
             headers=headers,
             debug=self.debug,
@@ -400,7 +402,7 @@ class GitHubHandler(BaseHookHandler):
             'pullrequest_ref': options.get('pullrequest_ref', None) or 'merge',
             'token': options.get('token', None),
             'debug': options.get('debug', None) or False,
-            'verify': options.get('verify', None) or False,
+            'verify': options.get('verify', None) or True,
         }
         handler = klass(options.get('secret', None), options.get('strict', False), **klass_kwargs)
         self.handler = handler
