@@ -13,13 +13,16 @@
 #
 # Copyright Buildbot Team Members
 
+from __future__ import annotations
+
 import calendar
 import datetime
 import os
 from binascii import hexlify
 
 import jwt
-
+import twisted
+from packaging.version import parse as parse_version
 from twisted.application import strports
 from twisted.cred.portal import IRealm
 from twisted.cred.portal import Portal
@@ -32,6 +35,7 @@ from twisted.web import resource
 from twisted.web import server
 from zope.interface import implementer
 
+from buildbot import config
 from buildbot.plugins.db import get_plugins
 from buildbot.util import bytes2unicode
 from buildbot.util import service
@@ -80,13 +84,16 @@ class BuildbotSession(server.Session):
 
     def _fromToken(self, token):
         try:
-            decoded = jwt.decode(token, self.site.session_secret, algorithms=[
-                                 SESSION_SECRET_ALGORITHM])
+            decoded = jwt.decode(
+                token, self.site.session_secret, algorithms=[SESSION_SECRET_ALGORITHM]
+            )
         except jwt.exceptions.ExpiredSignatureError as e:
             raise KeyError(str(e)) from e
         except jwt.exceptions.InvalidSignatureError as e:
-            log.msg(e, "Web request has been rejected."
-                    "Signature verification failed while decoding JWT.")
+            log.msg(
+                e,
+                "Web request has been rejected.Signature verification failed while decoding JWT.",
+            )
             raise KeyError(str(e)) from e
         except Exception as e:
             log.err(e, "while decoding JWT session")
@@ -109,17 +116,15 @@ class BuildbotSession(server.Session):
         else:
             cookieString = b"TWISTED_SECURE_SESSION"
 
-        cookiename = b"_".join([cookieString] + request.sitepath)
-        request.addCookie(cookiename, self.uid, path=b"/",
-                          secure=secure)
+        cookiename = b"_".join([cookieString, *request.sitepath])
+        request.addCookie(cookiename, self.uid, path=b"/", secure=secure)
 
     def expire(self):
         # caller must still call self.updateSession() to actually expire it
         self._defaultValue()
 
     def notifyOnExpire(self, callback):
-        raise NotImplementedError(
-            "BuildbotSession can't support notify on session expiration")
+        raise NotImplementedError("BuildbotSession can't support notify on session expiration")
 
     def touch(self):
         pass
@@ -130,20 +135,20 @@ class BuildbotSession(server.Session):
 
         This should actually only be used for cookie generation
         """
-        exp = datetime.datetime.utcnow() + self.expDelay
+        exp = datetime.datetime.now(datetime.timezone.utc) + self.expDelay
         claims = {
             'user_info': self.user_info,
             # Note that we use JWT standard 'exp' field to implement session expiration
             # we completely bypass twisted.web session expiration mechanisms
-            'exp': calendar.timegm(datetime.datetime.timetuple(exp))}
+            'exp': calendar.timegm(datetime.datetime.timetuple(exp)),
+        }
 
         return jwt.encode(claims, self.site.session_secret, algorithm=SESSION_SECRET_ALGORITHM)
 
 
 class BuildbotSite(server.Site):
-
-    """ A custom Site for Buildbot needs.
-        Supports rotating logs, and JWT sessions
+    """A custom Site for Buildbot needs.
+    Supports rotating logs, and JWT sessions
     """
 
     def __init__(self, root, logPath, rotateLength, maxRotatedFiles):
@@ -155,7 +160,8 @@ class BuildbotSite(server.Site):
     def _openLogFile(self, path):
         self._nativeize = True
         return LogFile.fromFullPath(
-            path, rotateLength=self.rotateLength, maxRotatedFiles=self.maxRotatedFiles)
+            path, rotateLength=self.rotateLength, maxRotatedFiles=self.maxRotatedFiles
+        )
 
     def getResourceFor(self, request):
         request.responseHeaders.removeHeader('Server')
@@ -183,7 +189,7 @@ class BuildbotSite(server.Site):
 
 
 class WWWService(service.ReconfigurableServiceMixin, service.AsyncMultiService):
-    name = 'www'
+    name: str | None = 'www'  # type: ignore[assignment]
 
     def __init__(self):
         super().__init__()
@@ -268,19 +274,29 @@ class WWWService(service.ReconfigurableServiceMixin, service.AsyncMultiService):
 
     def refresh_base_plugin_name(self, new_config):
         if 'base_react' in new_config.www.get('plugins', {}):
-            self.base_plugin_name = 'base_react'
-        else:
-            self.base_plugin_name = 'base'
+            config.error(
+                "'base_react' plugin is no longer supported. Use 'base' plugin in master.cfg "
+                "BuildmasterConfig['www'] dictionary instead. Remove 'buildbot-www-react' and "
+                "install 'buildbot-www' package."
+            )
+        self.base_plugin_name = 'base'
 
     def configPlugins(self, root, new_config):
         plugin_root = root
-        if self.base_plugin_name == 'base_react':
-            plugin_root = resource.NoResource()
-            root.putChild(b"plugins", plugin_root)
+        current_version = parse_version(twisted.__version__)
+        if current_version < parse_version("22.10.0"):
+            from twisted.web.resource import NoResource
+
+            plugin_root = NoResource()
+        else:
+            from twisted.web.pages import notFound
+
+            plugin_root = notFound()
+        root.putChild(b"plugins", plugin_root)
 
         known_plugins = set(new_config.www.get('plugins', {})) | set([self.base_plugin_name])
         for key, plugin in list(new_config.www.get('plugins', {}).items()):
-            log.msg(f"initializing www plugin {repr(key)}")
+            log.msg(f"initializing www plugin {key!r}")
             if key not in self.apps:
                 raise RuntimeError(f"could not find plugin {key}; is it installed?")
             app = self.apps.get(key)
@@ -288,33 +304,25 @@ class WWWService(service.ReconfigurableServiceMixin, service.AsyncMultiService):
             app.setConfiguration(plugin)
             plugin_root.putChild(unicode2bytes(key), app.resource)
 
-            if not app.ui:
-                del new_config.www['plugins'][key]
         for plugin_name in set(self.apps.names) - known_plugins:
-            log.msg(f"NOTE: www plugin {repr(plugin_name)} is installed but not configured")
+            log.msg(f"NOTE: www plugin {plugin_name!r} is installed but not configured")
 
     def setupSite(self, new_config):
         self.refresh_base_plugin_name(new_config)
 
         self.reconfigurableResources = []
 
-        # we're going to need at least the base plugin (buildbot-www)
+        # we're going to need at least the base plugin (buildbot-www or buildbot-www-react)
         if self.base_plugin_name not in self.apps:
             raise RuntimeError("could not find buildbot-www; is it installed?")
 
         root = self.apps.get(self.base_plugin_name).resource
         self.configPlugins(root, new_config)
         # /
-        if self.base_plugin_name == 'base':
-            root.putChild(b'', wwwconfig.IndexResource(
-                self.master,
-                self.apps.get(self.base_plugin_name).static_dir
-            ))
-        else:
-            root.putChild(b'', wwwconfig.IndexResourceReact(
-                self.master,
-                self.apps.get(self.base_plugin_name).static_dir
-            ))
+        root.putChild(
+            b'',
+            wwwconfig.IndexResource(self.master, self.apps.get(self.base_plugin_name).static_dir),
+        )
 
         # /auth
         root.putChild(b'auth', auth.AuthRootResource(self.master))
@@ -324,7 +332,7 @@ class WWWService(service.ReconfigurableServiceMixin, service.AsyncMultiService):
 
         # /api
         root.putChild(b'api', rest.RestRootResource(self.master))
-        [graphql]  # import is made for side effects
+        _ = graphql  # import is made for side effects
 
         # /config
         root.putChild(b'config', wwwconfig.ConfigResource(self.master))
@@ -341,23 +349,26 @@ class WWWService(service.ReconfigurableServiceMixin, service.AsyncMultiService):
         # FIXME: this does not work with reconfig
         change_hook_auth = new_config.www.get('change_hook_auth')
         if change_hook_auth is not None:
-            resource_obj = self.setupProtectedResource(
-                resource_obj, change_hook_auth)
+            resource_obj = self.setupProtectedResource(resource_obj, change_hook_auth)
         root.putChild(b"change_hook", resource_obj)
 
         self.root = root
 
-        rotateLength = new_config.www.get(
-            'logRotateLength') or self.master.log_rotation.rotateLength
-        maxRotatedFiles = new_config.www.get(
-            'maxRotatedFiles') or self.master.log_rotation.maxRotatedFiles
+        rotateLength = (
+            new_config.www.get('logRotateLength') or self.master.log_rotation.rotateLength
+        )
+        maxRotatedFiles = (
+            new_config.www.get('maxRotatedFiles') or self.master.log_rotation.maxRotatedFiles
+        )
 
         httplog = None
         if new_config.www['logfileName']:
             httplog = os.path.abspath(
-                os.path.join(self.master.basedir, new_config.www['logfileName']))
-        self.site = BuildbotSite(root, logPath=httplog, rotateLength=rotateLength,
-                                 maxRotatedFiles=maxRotatedFiles)
+                os.path.join(self.master.basedir, new_config.www['logfileName'])
+            )
+        self.site = BuildbotSite(
+            root, logPath=httplog, rotateLength=rotateLength, maxRotatedFiles=maxRotatedFiles
+        )
 
         self.site.sessionFactory = None
 
@@ -387,8 +398,7 @@ class WWWService(service.ReconfigurableServiceMixin, service.AsyncMultiService):
     @defer.inlineCallbacks
     def makeSessionSecret(self):
         state = self.master.db.state
-        objectid = yield state.getObjectId(
-            "www", "buildbot.www.service.WWWService")
+        objectid = yield state.getObjectId("www", "buildbot.www.service.WWWService")
 
         def create_session_secret():
             # Bootstrap: We need to create a key, that will be shared with other masters
@@ -397,14 +407,14 @@ class WWWService(service.ReconfigurableServiceMixin, service.AsyncMultiService):
             # we encode that in hex for db storage convenience
             return bytes2unicode(hexlify(os.urandom(int(SESSION_SECRET_LENGTH / 8))))
 
-        session_secret = yield state.atomicCreateState(objectid, "session_secret",
-                                                       create_session_secret)
+        session_secret = yield state.atomicCreateState(
+            objectid, "session_secret", create_session_secret
+        )
         self.site.setSessionSecret(session_secret)
 
     def setupProtectedResource(self, resource_obj, checkers):
         @implementer(IRealm)
         class SimpleRealm:
-
             """
             A realm which gives out L{ChangeHookResource} instances for authenticated
             users.
